%%
% Author: Chenzhe Qian
% Team: iGEM 2012 Calgary
% Copy Rights Reversed
%%

function decarbModel = decarboxylation(model)

% acyl-coa sythetase: chpa + accoa + atp <--> chp_scoa + amp + ppi
% acyl-coa acp transacylase: chp_scoa + acp <--> chp_sacp + coa
% acyl-acp reducatase: chp_sacp + nad(p)h + h <--> acp + nad(p) + chpo
% aldehyde decarbonylase: chpo + o2 + 2nadph + 2h <--> chp + for + h2o +
% 2nadp

rxnNamesList = {'CHPA exchange','CHPA transport',... 
                'Acyl-CoA synthetase', 'Acyl-CoA ACP transacylase',...
                'Acyl-ACP reducatase', 'Acyl-ACP reducatase P', 'Aldehyde decarbonylase',...
                'TempOutput_chp[c]'};
        
rxnMetsList = {{'chpa[e]'},{'chpa[e]', 'chpa[c]'},...
                {'chpa[c]', 'accoa[c]', 'atp[c]', 'chp_scoa[c]', 'amp[c]', 'ppi[c]'}, {'chp_scoa[c]','ACP[c]', 'chp_sacp[c]', 'coa[c]'},...
                {'chp_sacp[c]','nadh[c]','h[c]', 'ACP[c]','nad[c]','chpo[c]'}, {'chp_sacp[c]','nadph[c]', 'h[c]', 'nadp[c]','chpo[c]', 'ACP[c]'},...
                {'chpo[c]', 'o2[c]', 'nadph[c]', 'h[c]', 'chp[c]', 'for[c]', 'h2o[c]','nadp[c]'},...
                {'chp[c]'}};

stoichCoeff = {[-1], [-1 1], [-1 -1 -1 1 1 1], [-1 -1 1 1], [-1 -1 -1 1 1 1], [-1 -1 -1 1 1 1], [-1 -1 -2 -2 1 1 1 2], [-1]};
        
revFlagList = [1 1 1 1 1 1 1 0];
lbList = [-1000 -1000 -1000 -1000 -1000 -1000 -1000 0];
ubList = [1000 1000 1000 1000 1000 1000 1000 1000];

decarbModel = model;
for i = 1:length(rxnNamesList)
    decarbModel = addReaction(decarbModel,rxnNamesList{i}, rxnMetsList{i}, stoichCoeff{i}, revFlagList(i), lbList(i), ubList(i));
end

% testPathway(decarbModel,'chpa[e]','chp[c]');
