%%
% Author: Chenzhe Qian
% Team: iGEM 2012 Calgary
% Copy Rights Reversed
%%

function denitrModel = denitrification(model)

% Carbazole
% ABPD 2'-Aminobiphenyl-2,3-diol
% HOHD 2-Hydroxy-6-oxo-(2'-aminophenyl)-hexa-2E,4Z-dienoate
% HPD cis-2-Hydroxypenta-2,4-dienoate
% AB 2-Aminobenzoate

% carbazole 1,9a-dioxygenase: Carbazole + nad(p)h + o2 + h -->abpd + nad(p)
% 2'-aminobiphenyl-2,3-diol 1,2-dioxygenase: abpd + o2 --> hohd + h
% 2-hydroxy-6-oxo-6-(2'-aminophenyl)-hexa-2E,4Z-dienoate hydrolase: hohd + h2o --> ab + hpd + h
            
rxnNamesList = {'Carbazole exchange', 'Carbazole transport',...
                'carbazole dioxygenase','carbazole dioxygenase P',...
                'aminobipheny diol dioxygenase','hydroxyoxo(aminophenyl)hexadienoate hydrolase',...
                'TempOutput_AB[c]','TempOutput_HPD[c]'};
        
rxnMetsList = {{'Carbazole[e]'}, {'Carbazole[e]','Carbazole[c]'},...
                {'Carbazole[c]','nadh[c]','o2[c]','h[c]','ABPD[c]','nad[c]'}, {'Carbazole[c]','nadph[c]','o2[c]','h[c]','ABPD[c]','nadp[c]'},...
                {'o2[c]','ABPD[c]','HOHD[c]','h[c]'},{'h2o[c]','HOHD[c]','AB[c]', 'HPD[c]', 'h[c]'},...
                {'AB[c]'}, {'HPD[c]'}};

stoichCoeff = {[-1], [-1 1], [-1 -1 -1 -1 1 1], [-1 -1 -1 -1 1 1], [-1 -1 1 1], [-1 -1 1 1 1], [-1], [-1]};
                
revFlagList = [1 1 1 1 1 1 0 0];
lbList = [-1000 -1000 0 0 0 0 0 0];
ubList = [1000 1000 1000 1000 1000 1000 1000 1000];

denitrModel = model;
for i = 1:length(rxnNamesList)
    denitrModel = addReaction(denitrModel,rxnNamesList{i}, rxnMetsList{i}, stoichCoeff{i}, revFlagList(i), lbList(i), ubList(i));
end
 
%pathway test
% testPathway(denitrModel,'Carbazole[e]','AB[c]');
% testPathway(denitrModel,'Carbazole[e]','HPD[c]');
