%%
% Author: Chenzhe Qian
% Team: iGEM 2012 Calgary
% Copy Rights Reversed
%%

function desulfModel = desulfurization(model)

% DBT Dibenzothiophene
% DBTo Dibenzothiophene sulfoxide
% DBToo dibenzothiophene sulfone
% HBS 2'-hydroxybiphenyl-2-sulfinate
% HB 2-Hydroxybiphenyl

% DBT + O2 + NADH + H <--> DBTo + NAD + H2O
% DBTo + O2 + NADH + H <--> DBToo + NAD + H2O
% DBToo + 2NADH + O2 + H <--> HBS + 2NAD + H2O
% HBS + H2O <--> SO3 + HB + H

rxnNamesList = {'DBT exchange', 'DBT transport',...
                'dibenzothiophene monooxygenase', 'dibenzothiophene monooxygenase O',...
                'dibenzothiophene-5,5-dioxide monooxygenase', '2-hydroxybiphenyl-2-sulfinate sulfinolyase',...
                'TempOutput_HBP[c]'};
        
rxnMetsList = {{'DBT[e]'}, {'DBT[e]', 'DBT[c]'},...
                {'DBT[c]', 'o2[c]', 'nadh[c]', 'h[c]', 'DBTO[c]','nad[c]','h2o[c]'},{'DBTO[c]', 'o2[c]', 'nadh[c]', 'h[c]', 'DBTO2[c]','nad[c]','h2o[c]'},...
                {'DBTO2[c]','nadh[c]','o2[c]','h[c]','HBPS[c]','nad[c]','h2o[c]'}, {'HBPS[c]','h2o[c]','so3[c]','HBP[c]', 'h[c]'},...
                {'HBP[c]'}};

stoichCoeff = {[-1], [-1 1], [-1 -1 -1 -1 1 1 1], [-1 -1 -1 -1 1 1 1], [-1 -2 -1 -1 1 2 1], [-1 -1 1 1 1], [-1]};
                
revFlagList = [1 1 1 1 1 1 0];
lbList = [-1000 -1000 0 0 0 0 0];
ubList = [1000 1000 1000 1000 1000 1000 1000];

desulfModel = model;
for i = 1:length(rxnNamesList)
    desulfModel = addReaction(desulfModel,rxnNamesList{i}, rxnMetsList{i}, stoichCoeff{i}, revFlagList(i), lbList(i), ubList(i));
end
 
%pathway test
% testPathway(desulfModel,'DBT[e]','HBP[c]');
