%%
% Author: Chenzhe Qian
% Team: iGEM 2012 Calgary
% Source code are governed under the BioBrick?s Foundation and iGEM 2012 copyright policies
%%

%% main function 
function [effectors oriGlucose oriBiomass maxFlux sol funcF biomasses] = main(model,step,n,diffThreshold,zeroThreshold,negFlag)

%% run traditional optimization computation
h = waitbar(0,'Initializing the system...');
s = optimizeCbModel(model,'max',0,false); % add loop constraints 
waitbar(1,h);
close(h);

%% plot the relationship between biomass rate and target compound rate
[sol funcF biomasses]= plotBiomassAndTarget(model, step, n);
% fplot(funcF,[biomasses(1) biomasses(end)]);
% hold all
% plot(sol, funcF(sol),'Marker','o','Color',[1 0 0],'MarkerSize',10);

bioPercent = sol / s.x(model.c~=0) * 100;
% fprintf('solution is: %f\n',sol);
% fprintf('s.x is: %f\n',s.x(model.c~=0));
% fprintf('percent is: %f\n',bioPercent);

%% run the FVB test for original model
h = waitbar(0,'Analying the metabolic network and trying to find candidate compound...');
[minFlux maxFlux oriBiomass xMin xMax] = fluxVariabilityCus(model, bioPercent, 'max', model.rxns(end), false, false); 
oriGlucose = xMax(findRxnIDs(model,'EX_glc(e)'));
waitbar(1/3,h);

%% run reaction analysis
if(negFlag == false)
    metsList = rxnAnalysis(model,xMin,xMax,diffThreshold,zeroThreshold);
else
    metsList = rxnAnalysis(model,xMax,xMin,diffThreshold,zeroThreshold);
end
waitbar(2/3,h);

%% trace back to initial uptaken source
candidates = traceSource(model,metsList);
waitbar(3/3,h);
close(h);

%% test the effect of candidates
effectors = candidatesTest(model,candidates,bioPercent,maxFlux,oriBiomass,negFlag);

%% end of the function


