%%
% Author: Chenzhe Qian
% Team: iGEM 2012 Calgary
% Source code are governed under the BioBrick?s Foundation and iGEM 2012 copyright policies
%%

function [solution funcF biomasses] = plotBiomassAndTarget(model, step, n)

%% determine the relation between target compound flux rate(T) and biomass flux
maxFluxes = zeros(step,1);
biomasses = zeros(step,1);
optPercent = 100.0/step;

% make sure the degree is smaller than the number of docs
if n >= step;
    errordlg('Error: degree should be less than the number of data points','Error');
    
else
    h = waitbar(0,'Approximating biomass vs targart compound rate...');
    for i = 0:step
        [tempMinFlux tempMaxFlux tempBiomass] = fluxVariabilityCus(model, optPercent*i, 'max', model.rxns(end), false, false);
        maxFluxes(i+1) = tempMaxFlux;
        biomasses(i+1) = tempBiomass;
        waitbar(i/step,h);
    end
    
    % compute the function of T on B
    p = polyfit(biomasses,maxFluxes,n);
    
    % check if either biomasses or maxFluxes only contains zeros
    if(~any(biomasses))
        msgbox('Cell cannot grow');
        close(h);
    elseif (~any(maxFluxes))
        msgbox('Cell cannot produce the target compound of the pathway');
        close(h);
    end

%     disp(biomasses);
%     disp(maxFluxes);

    % generate a function between biomass and flux rate
    syms x y;
    y = 0;
    for i = 1:length(p)
        y = y + p(i)*x^(length(p)-i);
    end

    %plot approx. function
    funcF = matlabFunction(y);
%     fplot(funcF,[biomasses(1) biomasses(end)]);
%     hold all

    y = y*x; % used to maximize T*B

    dy = diff(y); % take derivative of y;
    y = matlabFunction(dy);
  
    solution = fsolve(y,0); % the optimal biomass to max T*B
    
    % correct the range of solution
    if (solution <= 0)
        solution = 0.1;
    elseif (solution > biomasses(end))
        solution = biomasses(end);
    end

%     fplot(y,[biomasses(1) biomasses(end)]);
%     hold all
%     plot(solution, funcF(solution),'Marker','o','Color',[1 0 0],'MarkerSize',10);


    % f = @(x)0;
    % for i = 1:length(p)
    %     f = @(x)(f(x) + p(i)*x.^(length(p)-i));
    % end
    % f = @(x)f(x)*x; % used to maximize T*B
    close(h);
end

