%%
% Author: Chenzhe Qian
% Team: iGEM 2012 Calgary
% Source code are governed under the BioBrick?s Foundation and iGEM 2012 copyright policies
%%

%% reaction analysis function 
function [finalList bidirectionalRxns unidirectional fb fu] = rxnAnalysis(model,minFlux,maxFlux,diffThreshold,zeroThreshold)

%% compute the difference of flux rate for each compound between two model with different objective sence
diffPercent = (maxFlux - minFlux)./maxFlux;

bidirectionalRxns = [];
% unidirectionalForward = [];
% unidirectionalReverse = [];
unidirectional = [];

biDirIndex = 1;
uniDirIndex = 1;
for i = 1:2383 %length(minFlux) get rid of new added rxns
   if (abs(minFlux(i)) >= zeroThreshold) || (abs(maxFlux(i) >= zeroThreshold))
        if minFlux(i)*maxFlux(i)<0
           bidirectionalRxns(biDirIndex) = i;
           biDirIndex = biDirIndex+1;
       
       elseif (abs(diffPercent(i)) >= diffThreshold) && (abs(diffPercent(i)) ~= inf)
           unidirectional(uniDirIndex) = i;
           uniDirIndex = uniDirIndex+1;
        end
   end
       
end

if or(bidirectionalRxns<=0, isempty(bidirectionalRxns))
%     disp('none bidirectionalRxn')
else
%     disp('bidirectionalRxns:');
    fb = printRxnFormulaCus(model,model.rxns(bidirectionalRxns),minFlux,maxFlux,diffPercent,false);
%     disp(length(bidirectionalRxns));
end

if or(unidirectional<=0, isempty(unidirectional))
%     disp('no rxn has such difference');
else
%     disp('unidirectionalRxns:');
    fu = printRxnFormulaCus(model,model.rxns(unidirectional),minFlux,maxFlux,diffPercent,false);
%     disp(length(unidirectional));
end

% rxnNames = strtok(fb);
rxnNames = {strtok(fb) strtok(fu)};
rxns = {fb fu};

%% category the interested rxns and score each compounds
exchangeRxns= {};
finalList = {};
netScore = {};
index = 1;
for i = 1:length(rxns)
    temp = rxns{i}; %fb and fu
    for j = 1:length(temp)
        %for all exchange rxns
        if ~isempty(strfind(temp{j},'EX_')) 
            exchangeRxns{end+1} = temp{j};
%             disp(temp{j});
            [doNeed met] = checkNeed(temp{j},-1);
            if doNeed
                finalList{end+1} = [met ' 1000']; 
            else
                finalList{end+1} = [met ' -1000'];
            end
            
%        for all transport rxns
%         elseif ~isempty(strfind(temp{j},'[p]')) && (~isempty(strfind(temp{j},'[e]')) || ~isempty(strfind(temp{j},'[c]')))
%             if (checkNeed(temp{j},1));
%                 disp(doNeed);
%             end

        %for the rest rxns
        else
           values = score(temp{j});
           % sum up individual score to compute net score
           for m=1:length(values)
               [name score] = strtok(values{m});
%                disp(name);
%                disp(strtok(finalList));
               f = find(ismember(strtok(finalList),name),1);           
               if ~isempty(f)                
                    [tempName preScore] = strtok(finalList{f});
%                   fprintf('name:%s   tempName:%s   preScore:%s\n',name, tempName,preScore);  
                    newScore = str2double(score) + str2double(preScore);
                    finalList{f} = [name ' ' num2str(newScore)];
%                     if strcmp(name,'q8[c]')
%                         disp(temp{j});
%                         fprintf('pre:%s current:%s new:%s\n',preScore, score, num2str(newScore));
%                     end
               else
                   finalList{end+1} = [name ' ' score];
               end
           end
        end  
        
    end
end

finalList = finalList';
finalList = simplifyMetsList(finalList);
disp(finalList);


%% following functions are helpers that called in above
%% helper function to determine the need of rxns
% all exchange rxns are type -1
% all other rxns are type 1
function [doNeed met] = checkNeed(rxn,rxnType)
[name remain] = strtok(rxn);
[min remain] = strtok(remain);
[max remain] = strtok(remain);
[ratio remain] = strtok(remain);
met = strtok(remain);
%     fprintf('max:%f ratio: %f', str2double(max), str2double(ratio));
if (str2double(ratio)>0 && str2double(max)>0)
    doNeed = 1*rxnType;
else
    doNeed = -1*rxnType;
end

    
%% helper function to score each compound in candidate rxn list
function compoundValues = score(rxn)
compoundValues = {};
token = {};
while true
    [str rxn] = strtok(rxn);
    if isempty(str),  break;  end
    token{end+1} = str;
end
% disp(token);
% name = token{1};
% min = token{2};
% max = token{3};
% ratio = token{4};
score = str2double(token{3}) - str2double(token{2});
reactants = {};
products = {};
isReact = 1;
for i = 5:length(token)
    %to select all compounds from the rxn
    %all compound with [e] is either exchange rxn or transport rxn
    if strcmp(token{i},'<=>') || strcmp(token{i},'->'), isReact = 0; end
    if (~isempty(strfind(token{i},'[c]')) || ~isempty(strfind(token{i},'[p]')))
%         disp(token{i});
        if isReact
            reactants{end+1} = token{i};
            compoundValues{end+1} = [token{i} ' ' num2str(score)];
        else
            products{end+1} = token{i};
            score = -1 * score;
            compoundValues{end+1} = [token{i} ' ' num2str(score)];
        end
    end
end
% disp(reactants);
% disp(products);
% disp(compoundValues);

%% helper to simplify the list by merging the same mets in different compartments
function newList = simplifyMetsList(metsList)
% get rid of compartment label
metsList = strrep(metsList,'[c]','');
metsList = strrep(metsList,'[p]','');
metsList = strrep(metsList,'[e]','');

% merge the values of same mets in different compartment
for i = 1:(length(metsList)-1)
    for j = (i+1):length(metsList)
        [metJ jValue] = strtok(metsList{j});
        [metI iValue] = strtok(metsList{i});
        if strcmp(metJ,metI) && ~isempty(metJ);
%             fprintf('i:%d j:%d met1:%s met2:%s \n',i,j,metsList{i}, metsList{j});
            newValue = str2double(iValue) + str2double(jValue);
            metsList{j} = [];
            metsList{i} = [metI ' ' num2str(newValue)];
%             fprintf('new value:%d new:%s \n',newValue, metsList{i});
        end
    end
end

% get rid of empty element in the list
newList = metsList(~cellfun('isempty',metsList));


%% end of function


